#!/bin/bash

function example
{
    echo "example: ./backup-sql.sh -b mysql -o /srv/backup/sql -d nextcloud -u nextcloud -w foobar -s localhost"
}

function usage
{
    echo "usage: ./backup-sql.sh -b database_manager -o outputdir -d dbname -w dbpassword -p port -s remote_server [--log logfile] [--mail]"
}

function help {
  usage
    echo -e "MANDATORY:"
    echo -e "  -b, --dbm  VAL  The database manager to use (mysql, mariadb or postgresql)"
    echo -e "  -o, --outputdir  VAL  The output directory where to store the dump\n"
    echo -e "OPTION:"
    echo -e "  -d, --database        The name of the database to dump. If not specified, all databases will be dumped"
    echo -e "  -u, --user   VAL  A username allowed to dump the database."
    echo -e "  -w, --password   VAL  Password of the user allowed to dump the database."
    echo -e "  -p, --port   VAL  The port used to connect to the database."
    echo -e "  -s, --server   VAL  Adress of the server. Defaults to localhost."
    echo -e "  -h,  --help             Prints this help\n"
  example
}

# Number of Mandatory args
margs=2

# Ensures that the number of passed args are at least equals
# to the declared number of mandatory args.
# It also handles the special case of the -h or --help arg.
function margs_precheck {
	if [ $2 ] && [ $1 -lt $margs ]; then
		if [ $2 == "--help" ] || [ $2 == "-h" ]; then
			help
			exit
		else
	    	usage
			example
	    	exit 1 # error
		fi
	fi
}

# Ensures that all the mandatory args are not empty
function margs_check {
	if [ $# -lt $margs ]; then
	    usage
	  	example
	    exit 1 # error
	fi
}

# Main
margs_precheck $# $1

dbm=
dbName=
backupdir=

maxNrOfBackups=7
server=

while [ "$1" != "" ];
do
    case $1 in
      -b | --dbm )
               shift
               dbm=$1
               ;;
      -o | --outputdir )
               shift
               backupdir=$1
               ;;
      -d | --database )
               shift
               dbName=$1
               ;;
      -w | --password )
               shift
               dbPassword=$1
               ;;
     -s | --server )
              shift
              server=$1
              ;;
     -u | --user )
              shift
              dbUser=$1
              ;;
        -p | --port )
                 shift
                 dbPort=$1
                 ;;
        -m | --mail )
                 shift
                 mail=$1
                 ;;
        --log )
                 shift
                 log=$1
                 ;;
        -h | --help )
		help
                exit
                ;;
        * )
                echo "$script: illegal option $1"
                usage
	            	example
					 exit 1 # error
                ;;
    esac
    shift
done

margs_check $dbm $backupdir

# set log path
if [ "$log" = "" ]; then
    log=$backupdir
fi

now=$(date +"%FT%H%M")
fileNamePrefix="sqldump_${dbm,,}_${dbName:-all}"
fileNameBackupDb="${fileNamePrefix}_${now^^}.sql"
logfile="$log/${fileNamePrefix}_${now^^}.log"
printf "Log File - " > $logfile
echo $now >> $logfile

# PID - Check script is launched only once at a time
PIDFILE="${backupdir}/${dbm,,}${dbName,,:-all}.pid"
if [ -f $PIDFILE ]; then
  PID=$(cat $PIDFILE)
  ps -p $PID > /dev/null 2>&1
  if [ $? -eq 0 ]
  then
    echo "Job is already running" >> $logfile
    exit 1
  else
    ## Process not found assume not running
    echo $$ > $PIDFILE
    if [ $? -ne 0 ]; then
      echo "Could not create PID file" >> $logfile
      exit 1
    fi
  fi
else
  echo $$ > $PIDFILE
  if [ $? -ne 0 ]; then
    echo "Could not create PID file" >> $logfile
    exit 1
  fi
fi

#
# Backup DB
#
if [ "${dbm,,}" = "mysql" ] || [ "${dbm,,}" = "mariadb" ]; then
      echo "Backup database (MySQL/MariaDB)..." >> $logfile

    if ! [ -x "$(command -v mysqldump)" ]; then
        echo "ERROR: MySQL/MariaDB not installed (command mysqldump not found)." >> $logfile
        echo "ERROR: No backup of database possible!" >> $logfile
    else
        	mysqldump --log-error=$logfile --single-transaction -h ${server:-localhost} -P ${dbPort:-3306}${user:+ -u ${dbUser}}${dbPassword:+ -p ${dbPassword}} "${dbName:---all-databases}" | gzip > "${backupdir}/${fileNameBackupDb}.gz"
	fi

    echo "Done" >> $logfile
    echo
elif [ "${dbm,,}" = "postgresql" ]; then
    echo "Backup database (PostgreSQL)..." >> $logfile

    if ! [ -x "$(command -v pg_dump)" ]; then
        echo "ERROR:PostgreSQL not installed (command pg_dump not found)." >> $logfile
        echo "ERROR: No backup of database possible!" >> $logfile
    else
        if [ -z $dbName ]; then
           	${dbPassword:+PGPASSWORD=$dbPassword} pg_dumpall${server:+ -h $server}${dbPort:+ -p $dbPort}${dbUser:+ -U $dbUser} 2> >(tee -a $logfile >&2) | gzip >  "${backupdir}/${fileNameBackupDb}.gz"
        else
        	${dbPassword:+PGPASSWORD=$dbPassword} pg_dump "${dbName}"${server:+ -h $server}${dbPort:+ -p $dbPort}${dbUser:+ -U $dbUser} 2> >(tee -a $logfile >&2) | gzip >  "${backupdir}/${fileNameBackupDb}.gz"
       fi

    fi
    echo "Done" >> $logfile
    echo
else
  echo "ERROR: This database manager is not supported!" >> $logfile
  help
fi
# send the result
# if [ "$mail" != "" ]; then
#     if [ $status -eq 0 ]; then
#         tail -n15 $logfile | mail -s "Backup SQL: ${server} SUCCESS" $mail
#     else
#         tail -n15 $logfile | mail -s "Backup SQL: ${server} FAILED" $mail
#   fi
# fi

#
# Delete old backups
#

find $backupdir -name "${fileNamePrefix}*" -mtime +$maxNrOfBackups -delete

# clean pid file
rm $PIDFILE
